import 'package:flutter_test/flutter_test.dart';
import 'package:routing_path/src/path_matcher.dart';

void main() {
  Uri uri({
    String scheme = 'nuapp',
    String host = 'some-host',
    String path,
    List<String> segments,
  }) =>
      Uri(scheme: scheme, host: host, path: path, pathSegments: segments);

  test('identical matches', () {
    final url = uri();
    expect(_matchUrl(url, url), <String, String>{});
  });

  test('scheme matches', () {
    expect(_matchUrl(uri(), uri(scheme: 'aaa')), isNull);
    expect(_matchUrl(uri(scheme: 'aaa'), uri()), isNull);
  });

  test('host matches', () {
    expect(_matchUrl(uri(), uri(host: 'aaa')), isNull);
    expect(_matchUrl(uri(host: 'aaa'), uri()), isNull);
  });

  test('simple path matches', () {
    expect(
      _matchUrl(
        uri(path: 'path'),
        uri(path: 'path'),
      ),
      <String, String>{},
    );
    expect(
      _matchUrl(
        uri(path: 'path/to/somewhere'),
        uri(path: 'path'),
      ),
      isNull,
    );
    expect(
      _matchUrl(
        uri(path: 'path'),
        uri(path: 'path/to/somewhere'),
      ),
      isNull,
    );
  });

  test('replaceable path matches', () {
    expect(
      _matchUrl(uri(), uri(path: ':id')),
      isNull,
    );
    expect(
      _matchUrl(uri(path: ':id'), uri()),
      isNull,
    );
    expect(
      _matchUrl(uri(path: '123'), uri(path: ':id')),
      isNull,
    );
    expect(
      _matchUrl(uri(path: ':id'), uri(path: '123')),
      <String, String>{'id': '123'},
    );

    expect(
      _matchUrl(
        uri(path: ':id/:name/:value'),
        uri(path: '123/path-name/456-789'),
      ),
      <String, String>{
        'id': '123',
        'name': 'path-name',
        'value': '456-789',
      },
    );
    expect(
      _matchUrl(
        uri(path: 'path/:name/somewhere'),
        uri(path: 'path/path-name'),
      ),
      isNull,
    );

    expect(
      _matchUrl(
        uri(path: 'path/:id/:name'),
        uri(path: 'path/123/path-name'),
      ),
      <String, String>{
        'id': '123',
        'name': 'path-name',
      },
    );
    expect(
      _matchUrl(
        uri(path: 'path/:id/:name'),
        uri(path: 'path/123/path-name'),
      ),
      <String, String>{
        'id': '123',
        'name': 'path-name',
      },
    );

    expect(
      _matchUrl(
        uri(path: 'path/:id/to/:name'),
        uri(path: 'path/123/path-name'),
      ),
      isNull,
    );
  });
}

Map<String, String> _matchUrl(Uri match, Uri url) {
  if (url.scheme != match.scheme || url.host != match.host) {
    return null;
  }

  final pattern = buildPathPattern(match.path);
  return pathMatches(pattern, url.path);
}
